﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Optimization.Operators;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.TestFunctions {
  /// <summary>
  /// An operator that performs similarity calculation between two test functions solutions.
  /// </summary>
  [Item("SingleObjectiveTestFunctionSimilarityCalculator", "An operator that performs similarity calculation between two test functions solutions.")]
  [StorableClass]
  public sealed class SingleObjectiveTestFunctionSimilarityCalculator : SimilarityCalculator {
    #region Properties
    [Storable]
    public DoubleMatrix Bounds { get; set; }
    #endregion

    [StorableConstructor]
    private SingleObjectiveTestFunctionSimilarityCalculator(bool deserializing) : base(deserializing) { }
    private SingleObjectiveTestFunctionSimilarityCalculator(SingleObjectiveTestFunctionSimilarityCalculator original, Cloner cloner)
      : base(original, cloner) {
      this.Bounds = cloner.Clone(original.Bounds);
    }
    public SingleObjectiveTestFunctionSimilarityCalculator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SingleObjectiveTestFunctionSimilarityCalculator(this, cloner);
    }

    public static double CalculateSimilarity(RealVector left, RealVector right, DoubleMatrix bounds) {
      if (left == null || right == null)
        throw new ArgumentException("Cannot calculate similarity because one of the provided solutions or both are null.");
      if (bounds == null)
        throw new ArgumentException("Cannot calculate similarity because no bounds were provided.");
      if (left == right) return 1.0;

      double maxSum = 0.0;
      for (int i = 0; i < left.Length; i++)
        maxSum += Math.Pow(bounds[0, 0] - bounds[0, 1], 2);
      double maxDistance = Math.Sqrt(maxSum) / left.Length;

      double sum = 0.0;
      for (int i = 0; i < left.Length; i++)
        sum += Math.Pow(left[i] - right[i], 2);
      double distance = Math.Sqrt(sum) / left.Length;

      return 1.0 - distance / maxDistance;
    }

    public override double CalculateIndividualSimilarity(IScope left, IScope right) {
      var sol1 = left.Variables[Target].Value as RealVector;
      var sol2 = right.Variables[Target].Value as RealVector;

      return CalculateSimilarity(sol1, sol2, Bounds);
    }
  }
}
