﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Optimization.Operators;

namespace HeuristicLab.Problems.TravelingSalesman {
  /// <summary>
  /// An operator that performs similarity calculation between two traveling salesman solutions.
  /// </summary>
  [Item("TSPSimilarityCalculator", "An operator that performs similarity calculation between two traveling salesman solutions.")]
  public sealed class TSPSimilarityCalculator : SimilarityCalculator {
    private TSPSimilarityCalculator(bool deserializing) : base(deserializing) { }
    private TSPSimilarityCalculator(TSPSimilarityCalculator original, Cloner cloner) : base(original, cloner) { }
    public TSPSimilarityCalculator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new TSPSimilarityCalculator(this, cloner);
    }

    public static double CalculateSimilarity(Permutation left, Permutation right) {
      if (left == null || right == null)
        throw new ArgumentException("Cannot calculate similarity because one of the provided solutions or both are null.");
      if (left == right) return 1.0;

      int[,] edges = new int[right.Length, 2];
      for (int i = 0; i < right.Length; i++) {
        edges[right[i], 0] = right[(i + 1) % right.Length];
        edges[right[i], 1] = right[(i - 1 + right.Length) % right.Length];
      }

      double similarity = 0.0;
      for (int i = 0; i < left.Length; i++) {
        int targetCity = left[(i + 1) % left.Length];
        if (targetCity == edges[left[i], 0] || targetCity == edges[left[i], 1])
          similarity++;
      }

      return similarity / left.Length;
    }

    public override double CalculateIndividualSimilarity(IScope left, IScope right) {
      var sol1 = left.Variables[Target].Value as Permutation;
      var sol2 = right.Variables[Target].Value as Permutation;

      return CalculateSimilarity(sol1, sol2);
    }
  }
}
